from functools import wraps
from time import time, sleep


class RateLimiter:
    def __init__(self, rate_per_second):
        self.min_interval = 1 / float(rate_per_second)
        self.last_call = time()

    def __call__(self, fun):
        @wraps(fun)
        def wrapper(*args, **kwargs):
            self._limit_rate()
            fun(*args, **kwargs)
        return wrapper

    def _limit_rate(self):
        since_last_call = time() - self.last_call
        to_next_call = self.min_interval - since_last_call
        self.last_call = time()
        if to_next_call > 0:
            sleep(to_next_call)
