import json
from tornado.websocket import WebSocketHandler

from tfw.networking.serialization import deserialize_all
from tfw.networking.server.event_handler_connector import EventHandlerConnector
from tfw.config.logs import logging
log = logging.getLogger(__name__)


class ZMQWebSocketHandler(WebSocketHandler):
    _event_handler_connector = EventHandlerConnector()
    instances = set()

    def prepare(self):
        ZMQWebSocketHandler.instances.add(self)

    def on_close(self):
        ZMQWebSocketHandler.instances.remove(self)

    def open(self, *args, **kwargs):
        log.debug('WebSocket connection initiated')
        self._event_handler_connector.register_callback(self.zmq_callback)

    @staticmethod
    def zmq_callback(msg_parts):
        key, data = deserialize_all(*msg_parts)
        log.debug('Received on pull socket: {}'.format(data))
        for instance in ZMQWebSocketHandler.instances:
            instance.write_message(data)

    def on_message(self, message):
        log.debug('Received on WebSocket: {}'.format(message))
        self.send_message(self.make_response(message))

    def make_response(self, message):
        raise NotImplementedError

    def send_message(self, message: dict, key: str = None):
        self._event_handler_connector.send_message(message, key)

    # much secure, very cors, wow
    def check_origin(self, origin):
        return True


class ZMQWebSocketProxy(ZMQWebSocketHandler):
    def initialize(self, make_response, proxy_filter):
        self._make_response = make_response
        self._proxy_filter = proxy_filter

    def on_message(self, message):
        message = json.loads(message)
        if self._proxy_filter(message):
            super().on_message(message)

    def make_response(self, message):
        return self._make_response(message)
